import argparse
import os

from pyrpkg.cli import cliClient
from pyrpkg import utils

from exceptions import NotAnUpstreamException

class rpkgClient(cliClient):
    def __init__(self, config, name=None):
        self.DEFAULT_CLI_NAME = 'rpkg'
        super(rpkgClient, self).__init__(config, name)

    def setup_argparser(self):
        """Setup the argument parser and register some basic commands."""

        self.parser = argparse.ArgumentParser(
            prog=self.name,
            epilog='For detailed help pass --help to a target')

        # Add some basic arguments that should be used by all.
        # Add a config file
        self.parser.add_argument('--config', '-C',
                                 default=None,
                                 help='Specify a config file to use')
        # Allow forcing the package name
        self.parser.add_argument('--module-name',
                                 help=('Override the module name. Otherwise'
                                       ' it is discovered from: Git push URL'
                                       ' or Git URL (last part of path with'
                                       ' .git extension removed) or from name'
                                       ' macro in spec file. In that order.')
                                 )
        # Override the  discovered user name
        self.parser.add_argument('--user', default=None,
                                 help='Override the discovered user name')
        # Let the user define a path to work in rather than cwd
        self.parser.add_argument('--path', default=None,
                                 type=utils.u,
                                 help='Define the directory to work in '
                                 '(defaults to cwd)')
        # Verbosity
        self.parser.add_argument('--verbose', '-v', dest='v',
                                 action='store_true',
                                 help='Run with verbose debug output')
        self.parser.add_argument('--debug', '-d', dest='debug',
                                 action='store_true',
                                 help='Run with debug output')
        self.parser.add_argument('-q', action='store_true',
                                 help='Run quietly only displaying errors')

    def setup_subparsers(self):
        """Setup basic subparsers that all clients should use"""

        # Setup some basic shared subparsers

        # help command
        self.register_help()

        # Add a common parsers
        self.register_rpm_common()

        # Other targets
        self.register_make_source()
        self.register_clean()
        self.register_clog()
        self.register_clone()
        self.register_copr_build()
        self.register_commit()
        self.register_compile()
        self.register_diff()
        self.register_gimmespec()
        self.register_giturl()
        self.register_import_srpm()
        self.register_install()
        self.register_lint()
        self.register_local()
        self.register_new()
        self.register_new_sources()
        self.register_patch()
        self.register_prep()
        self.register_pull()
        self.register_push()
        self.register_sources()
        self.register_srpm()
        self.register_switch_branch()
        self.register_tag()
        self.register_unused_patches()
        self.register_upload()
        self.register_verify_files()
        self.register_verrel()

    def load_cmd(self):
        """This sets up the cmd object"""

        # load items from the config file
        items = dict(self.config.items(self.name, raw=True))

        # Read comma separated list of kerberos realms
        realms = [realm
                  for realm in items.get("kerberos_realms", '').split(',')
                  if realm]

        # Create the cmd object
        self._cmd = self.site.Commands(self.args.path,
                                       items['lookaside'],
                                       items['lookasidehash'],
                                       items['lookaside_cgi'],
                                       items['gitbaseurl'],
                                       items['anongiturl'],
                                       branchre='.*',
                                       kojiconfig='',
                                       build_client=None,
                                       user=self.args.user,
                                       quiet=self.args.q,
                                       realms=realms
                                       )

        self._cmd.module_name = self.args.module_name
        self._cmd.debug = self.args.debug
        self._cmd.verbose = self.args.v
        self._cmd.clone_config = items.get('clone_config')

    def register_make_source(self):
        make_source_parser = self.subparsers.add_parser(
            'make-source', help='Create Source0 from the '
            'content of the current working directory. '
            'The content must be of upstream type.',
            description='Puts content of the current '
            'working directory into a gzip-compressed archive '
            'named according to Source0 filename as specfied in '
            'the .spec file. The content must be of upstream '
            'type, otherwise no action is taken. Upstream '
            'content is such that it contains a .spec file '
            'that references no present source or patch '
            '(typically it contains only Source0 being '
            'generated automatically) and there is at least '
            'one file not in the list of ignored content (README, '
            'README.md, sources, tito.props, hidden files, and '
            'the .spec file itself).')
        make_source_parser.add_argument(
            '--outdir', default=os.getcwd(),
            help='Where to put the generated source. '
            'By default cwd.')
        make_source_parser.set_defaults(command=self.make_source)

    def make_source(self):
        try:
            self.cmd.make_source(self.args.outdir)
        except NotAnUpstreamException as e:
            self.log.error(str(e))

    def srpm(self):
        self.sources()
        try:
            self.cmd.make_source(os.getcwd())
        except NotAnUpstreamException:
            pass
        self.cmd.srpm(hashtype=self.args.hash)

    def register_srpm(self):
        """Register the srpm target"""
        srpm_parser = self.subparsers.add_parser(
            'srpm', help='Create a source rpm',
            description='Create a source rpm out of '
            'downstream or upstream content. See '
            'make-sources for the description of the '
            'two content types and their recognition.')
        # optionally define old style hashsums
        srpm_parser.add_argument(
            '--md5', action='store_const', const='md5', default=None,
            dest='hash', help='Use md5 checksums (for older rpm hosts)')
        srpm_parser.set_defaults(command=self.srpm)
