import unittest
import mock
import tempfile
import os
import rpm
import shutil
import tarfile

from string import Template

import rpkglib
from rpkglib.exceptions import NotAnUpstreamException, RpmSpecParseException,\
        NoSourceZeroException
from rpkglib.utils import find_source_zero

SPEC_TEMPLATE = Template("""
Name:       testpkg
Version:    1
Release:    1%{?dist}
Summary:    This is a test package.

License:    GPLv2+
URL:        https://someurl.org
Source0:    $source0

%description
""")

SPEC_WITH_PATCH_TEMPLATE = Template("""
Name:       testpkg
Version:    1
Release:    1%{?dist}
Summary:    This is a test package.

License:    GPLv2+
URL:        https://someurl.org
Source0:    $source0

Patch0:     $patch0

%description
""")

INVALID_SPEC = """
Name:       testpkg
Version:    1
Release:    1%{?dist}
Summary:    This is a test package.

License:    GPLv2+
URL:        https://someurl.org

Source0:    $source0
"""

NO_SOURCE_ZERO_SPEC = """
Name:       testpkg
Version:    1
Release:    1%{?dist}
Summary:    This is a test package.

License:    GPLv2+
URL:        https://someurl.org

%description
"""

class TestCommands(unittest.TestCase):
    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()
        self.cmd = rpkglib.Commands(self.tmpdir,
                                    'lookaside',
                                    'lookasidehash',
                                    'lookaside_cgi',
                                    'gitbaseurl',
                                    'anongiturl',
                                    branchre='.*',
                                    kojiconfig='',
                                    build_client=None)

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def dump_spec(self, spec_content):
        spec_path = os.path.join(self.tmpdir, 'testpkg.spec')
        spec_file = open(spec_path, 'w')
        spec_file.write(spec_content)
        spec_file.close()
        return spec_path

    def touch_file(self, filename, subdir=None):
        if subdir:
            dirpath = os.path.join(self.tmpdir, subdir)
            os.makedirs(dirpath)
        else:
            dirpath = self.tmpdir
        filepath = os.path.join(dirpath, filename)
        open(filepath, 'w').close()
        return filepath

    def get_parsed_spec(self, spec_path):
        ts = rpm.ts()
        rpm.addMacro("_sourcedir", self.tmpdir)
        return ts.parseSpec(spec_path)

    def test_is_upstream_source_is_present(self):
        spec_content = SPEC_TEMPLATE.substitute(
            source0='source0.tar.gz', patch0='patch0.txt')
        spec_path = self.dump_spec(spec_content)
        self.touch_file('source0.tar.gz')
        parsed_spec = self.get_parsed_spec(spec_path)
        is_upstream = self.cmd.is_upstream(self.tmpdir, parsed_spec.sources)
        self.assertFalse(is_upstream)

    def test_is_upstream_source_not_present(self):
        spec_content = SPEC_TEMPLATE.substitute(
            source0='source0.tar.gz', patch0='patch0.txt')
        spec_path = self.dump_spec(spec_content)
        self.touch_file('source1.tar.gz')
        parsed_spec = self.get_parsed_spec(spec_path)
        is_upstream = self.cmd.is_upstream(self.tmpdir, parsed_spec.sources)
        self.assertTrue(is_upstream)

    def test_is_upstream_patch_present(self):
        spec_content = SPEC_WITH_PATCH_TEMPLATE.substitute(
            source0='source0.tar.gz', patch0='patch.txt')
        spec_path = self.dump_spec(spec_content)
        self.touch_file('patch.txt')
        parsed_spec = self.get_parsed_spec(spec_path)
        is_upstream = self.cmd.is_upstream(self.tmpdir, parsed_spec.sources)
        self.assertFalse(is_upstream)

    def test_is_upstream_ignored_files(self):
        spec_content = SPEC_TEMPLATE.substitute(source0='source0.tar.gz')
        spec_path = self.dump_spec(spec_content)
        parsed_spec = self.get_parsed_spec(spec_path)
        is_upstream = self.cmd.is_upstream(self.tmpdir, parsed_spec.sources)
        self.assertFalse(is_upstream)

        ignored_filenames = [
            'README', 'readme', 'README.md',
            'tito.props', 'x.spec', 'sources', '.hiddden'
        ]

        for filename in ignored_filenames:
            self.touch_file(filename)

        is_upstream = self.cmd.is_upstream(self.tmpdir, parsed_spec.sources)
        self.assertFalse(is_upstream)

        self.touch_file('non-ignored-filename')
        is_upstream = self.cmd.is_upstream(self.tmpdir, parsed_spec.sources)
        self.assertTrue(is_upstream)

        self.touch_file('source0.tar.gz')
        is_upstream = self.cmd.is_upstream(self.tmpdir, parsed_spec.sources)
        self.assertFalse(is_upstream)

    def test_make_source_raises_on_downstream(self):
        spec_content = SPEC_TEMPLATE.substitute(source0='source0.tar.gz')
        self.dump_spec(spec_content)
        with self.assertRaises(NotAnUpstreamException):
            self.cmd.make_source(self.tmpdir)

    def test_make_source_packs_on_upstream(self):
        spec_content = SPEC_TEMPLATE.substitute(source0='source0.tar.gz')
        spec_path = self.dump_spec(spec_content)
        self.touch_file('patch.txt', subdir='dir')
        archive_path = self.cmd.make_source(self.tmpdir)
        self.assertTrue(os.path.exists(archive_path))

        parsed_spec = self.get_parsed_spec(spec_path)
        source0_filename = find_source_zero(parsed_spec.sources)
        self.assertEquals(os.path.basename(archive_path), source0_filename)

        parsed_spec = self.get_parsed_spec(spec_path)
        source0_filename = find_source_zero(parsed_spec.sources)
        self.assertEquals(os.path.basename(archive_path), source0_filename)

        expected_names = sorted([
            'testpkg-1', 'testpkg-1/dir', 'testpkg-1/dir/patch.txt', 'testpkg-1/testpkg.spec'
        ])
        tarball = tarfile.open(archive_path, 'r:gz')
        self.assertEquals(expected_names, sorted(tarball.getnames()))

    def test_make_source_raises_on_invalid_spec(self):
        spec_content = INVALID_SPEC
        self.dump_spec(spec_content)
        with self.assertRaises(RpmSpecParseException):
            self.cmd.make_source(self.tmpdir)

    def test_make_source_raises_on_no_source_zero(self):
        spec_content = NO_SOURCE_ZERO_SPEC
        self.dump_spec(spec_content)
        self.touch_file('patch.txt')
        with self.assertRaises(NoSourceZeroException):
            self.cmd.make_source(self.tmpdir)
